/***************************************************************************
 *   Copyright (C) 2004 by Christoph                                       *
 *   chris@inferno.nadir.org                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "kletterwizard.h"
#include "klwSettings.h"
#include "klwpersonalsettingsform.h"
#include "klwlayoutform.h"
#include "klwbinariesform.h"
#include "klwheaderform.h"

#include <qlabel.h>
#include <qlistview.h>
#include <qlineedit.h>
#include <qtextedit.h>
#include <qcheckbox.h>
#include <qcursor.h>
#include <qpopupmenu.h>
#include <kmenubar.h>

#include <kdebug.h>
#include <kmainwindow.h>
#include <klocale.h>

#include <kaccel.h>
#include <kstatusbar.h>
#include <ktextedit.h>
#include <kdatewidget.h>
#include <kconfigdialog.h>
#include <kprinter.h>
#include <klocale.h>


kletterwizard::kletterwizard(KURL url)
    : KParts::MainWindow(0,"kletterwizard")
    {
    setup();
    letter->letterOpen(url);
}

kletterwizard::kletterwizard()
    : KParts::MainWindow( 0, "kletterwizard" )
{
    setup();
}

void kletterwizard::setup()
{
    // set the shell's ui resource file
    setXMLFile("kletterwizardui.rc");
    setStandardToolBarMenuEnabled(TRUE);
    
    layout = new klwLayout(this,"Main Widget");
    letter = new klwLetter(this,"Letter");
    
    KStdAction::quit(this,SLOT(close()),actionCollection());
    KStdAction::preferences(this,SLOT(showSettings()),actionCollection());
    KStdAction::print(this,SLOT(print()),actionCollection());
    KStdAction::save(letter,SLOT(letterSave()),actionCollection());
    KStdAction::saveAs(letter,SLOT(letterSaveAs()),actionCollection());
    KStdAction::open(letter,SLOT(letterOpen()),actionCollection());
    //KStdAction::openRecent(letter,SLOT(letterOpen()),actionCollection());
    KStdAction::showMenubar(this,SLOT(slotToggleMenuBar()),actionCollection());
    
    savePSAction = new KAction(i18n("Save as &Postscript"),0,letter, SLOT(letterSavePS()), actionCollection(), "file_save_ps");
    saveTexAction = new KAction(i18n("Save as &Tex"),0,letter,SLOT(letterSaveTex()),actionCollection(),"file_save_tex");
    

    connect(layout->address->lineEditTo,SIGNAL(textChanged(const QString&)),letter,SLOT(letterNewTo(const QString&)));
    connect(layout->address->lineEditName,SIGNAL(textChanged(const QString&)),letter,SLOT(letterNewName(const QString&)));
    connect(layout->address->lineEditStreet,SIGNAL(textChanged(const QString&)),letter,SLOT(letterNewStreet(const QString&)));
    connect(layout->address->lineEditCity,SIGNAL(textChanged(const QString&)),letter,SLOT(letterNewCity(const QString&)));
    connect(layout->lineEditYourSign,SIGNAL(textChanged(const QString&)),letter,SLOT(letterNewYourSign(const QString&)));
    connect(layout->lineEditMySign,SIGNAL(textChanged(const QString&)),letter,SLOT(letterNewMySign(const QString&)));
    connect(layout->lineEditEnclosures,SIGNAL(textChanged(const QString&)),letter,SLOT(letterNewEnclosures(const QString&)));
    connect(layout->lineEditRegards,SIGNAL(textChanged(const QString&)),letter,SLOT(letterNewRegards(const QString& )));
    connect(layout->lineEditSalutation,SIGNAL(textChanged(const QString&)),letter,SLOT(letterNewSalutation(const QString& )));
    connect(layout->kDateYourLetterFrom,SIGNAL(changed(QDate)),letter,SLOT(letterNewYourLetterFrom(QDate)));
    connect(layout->lineEditSubject,SIGNAL(textChanged(const QString&)),letter,SLOT(letterNewSubject(const QString&)));
    connect(layout->chkYourLetterFrom,SIGNAL(stateChanged(int)),letter,SLOT(letterChkYourLetterFromChanged(int)));
    connect(layout->textEditText,SIGNAL(textChanged()),this,SLOT(layoutTextChanged()));
    connect(layout,SIGNAL(needGUIUpdating(bool )),this,SLOT(updateGUI(bool)));
    connect(layout,SIGNAL(needNewPS()),letter,SLOT(letterMakePS()));
    connect(letter,SIGNAL(letterPSNew(const KURL&)),layout,SLOT(openPS(const KURL&)));
    connect(letter,SIGNAL(letterLetterOpened(const klwLetter*)),layout,SLOT(letterOpened(const klwLetter*)));
    connect(letter,SIGNAL(letterBuildingStart()),this,SLOT(buildingStart()));
    connect(letter,SIGNAL(letterBuildingEnd()),this,SLOT(buildingEnd()));
    connect(letter,SIGNAL(fileSaved()),this,SLOT(fileSaved()));
    connect(layout->preview_viewer,SIGNAL(completed()),this,SLOT(updateCaption()));
    
    setCentralWidget(layout);
    statusBar()->show();
    statusBar()->message("Ready");
    createGUI(0L);
    
    if(actionCollection()->action(KStdAction::stdName(KStdAction::Print)) != NULL)
       actionCollection()->action(KStdAction::stdName(KStdAction::Print))->setEnabled(FALSE);
    if(actionCollection()->action(KStdAction::stdName(KStdAction::Save)) != NULL)
       actionCollection()->action(KStdAction::stdName(KStdAction::Save))->setEnabled(FALSE);
    if(actionCollection()->action(KStdAction::stdName(KStdAction::SaveAs)) != NULL)
       actionCollection()->action(KStdAction::stdName(KStdAction::SaveAs))->setEnabled(FALSE);
    savePSAction->setEnabled(FALSE);
    saveTexAction->setEnabled(FALSE);
    
}

kletterwizard::~kletterwizard()
{
}

void kletterwizard::layoutTextChanged()
{
 letter->letterNewText(layout->textEditText->text());  
}

void kletterwizard::updateGUI(bool b)
{
    if (b)
    {
	createGUI(layout->preview_viewer);
    }
    else
    {
	createGUI(0L);
    }
}

void kletterwizard::updateCaption()
{
	setCaption(letter->lmlFile.name());
}
void kletterwizard::showSettings()
{
    if (KConfigDialog::showDialog("klwSettings"))
    {
	return;
    }
    
    KConfigDialog *dialog = new KConfigDialog(this,"klwSettings",klwSettings::self());
    dialog->addPage(new klwPersonalSettingsForm(0,"PersonalForm"),i18n("Personal Settings"),"identity",i18n("Personal Settings"));
    dialog->addPage(new klwLayoutForm(0,"LayoutForm"),i18n("Layout"),"text_left",i18n("Layout"));
    dialog->addPage(new klwBinariesForm(0,"BinariesForm"),i18n("Binaries"),"misc",i18n("Binaries"));
    dialog->addPage(new klwHeaderForm(0,"HeaderForm"),i18n("Header"),"exec",i18n("Header"));
    connect(dialog,SIGNAL(settingsChanged()),this,SLOT(settingsChanged()));
    dialog->show();
    
    
}

void kletterwizard::settingsChanged()
{
    letter->somethingChanged();
    if (layout->currentPage == 2)
	letter->letterMakePS();
    layout->settingsChanged();
}

void kletterwizard::buildingStart()
{
    setCursor(QCursor(3));
    statusBar()->message(i18n("Building PostScript"));
}

void kletterwizard::buildingEnd()
{
    setCursor(QCursor(0));
    statusBar()->message(i18n("Building done"));
    if(actionCollection()->action(KStdAction::stdName(KStdAction::Print)) != NULL)
       actionCollection()->action(KStdAction::stdName(KStdAction::Print))->setEnabled(TRUE);
    if(actionCollection()->action(KStdAction::stdName(KStdAction::Save)) != NULL)
       actionCollection()->action(KStdAction::stdName(KStdAction::Save))->setEnabled(TRUE);
    if(actionCollection()->action(KStdAction::stdName(KStdAction::SaveAs)) != NULL)
       actionCollection()->action(KStdAction::stdName(KStdAction::SaveAs))->setEnabled(TRUE);
    savePSAction->setEnabled(TRUE);
    saveTexAction->setEnabled(TRUE);

}

void kletterwizard::print()
{
    letter->letterMakePS();
    if(letter->letterPSfile->exists())
    {
	KPrinter printer;
	if (printer.setup(this))
	{
	    printer.printFiles(letter->letterPSfile->name());
	}
    }
}

void kletterwizard::slotToggleMenuBar()
{
    if ( menuBar()->isVisible())
	menuBar()->hide();
    else
	menuBar()->show();
}

void kletterwizard::fileSaved()
{
    statusBar()->message(i18n("File Saved successfully"));
    updateCaption();
}

#include "kletterwizard.moc"


