/***************************************************************************
 *   Copyright (C) 2004 by Christoph                                       *
 *   chris@inferno.nadir.org                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "klwlayout.h"
#include "klwSettings.h"

#include <qcheckbox.h>
#include <qlineedit.h>
#include <qdatetime.h>
#include <qtabwidget.h>
#include <qgroupbox.h>

#include <kdatewidget.h>
#include <kactioncollection.h>
#include <ktextedit.h>
#include <kdebug.h>
#include <kxmlguifactory.h>


klwLayout::klwLayout(QWidget *parent, const char *name)
        : klwLayoutBase(parent, name)
{
    connect(chkYourLetterFrom,SIGNAL(stateChanged(int)),this,SLOT(toggleDate(int)));
    connect(tabContainer,SIGNAL(currentChanged(QWidget*)),this,SLOT(handleTabChange(QWidget*)));
    kDateYourLetterFrom->setDate(QDate::currentDate());
    lineEditSalutation->setText(klwSettings::salutation());
    lineEditRegards->setText(klwSettings::regards());
    textEditText->setCheckSpellingEnabled(TRUE);
    
    previewIsUptodate = FALSE;
    PSFile ="";
    
    //Request a KPart for the preview_viewer:
    KTrader::OfferList offers = KTrader::self()->query("image/x-eps","'KParts/ReadOnlyPart' in ServiceTypes");
    KLibFactory *factory = 0;
    KService::Ptr ptr = *offers.begin();
    factory = KLibLoader::self()->factory(ptr->library());
    if (factory )
    {
        preview_viewer = static_cast<KParts::ReadOnlyPart *>(factory->create(klwLayoutPreviewPage,ptr->name(),"KParts::ReadOnlyPart"));
        klwLayoutPreviewPageLayout->addWidget(preview_viewer->widget(),0,0,0);
    }
    else
    {
        kdFatal() << "Could NOT find a Part for image/x-eps (Postscript Viewer). Is KGhostview installed ?" << endl;
    }   
    
    
    //insert dropEnabled Address-Widget
    address = new klwLayoutAddress(groupBoxAddress);
    groupBoxAddressLayout->addWidget(address,0,0,0);
    
    //Disable some Actions from the KGhostview-Part
    KAction *previewAction;
    QStringList actions;
    actions << "readUp" << "watch_file" << "goToStart" << "media_menu" << "show_page_labels" << "toggle";
    actions << "orientation_menu" << "readDown" << "markEven" << "nextPage" ;
    actions << "prevPage" << "goToEnd" << "file_save_as" << "remove" << "toggle";
    for (QStringList::Iterator actionIt = actions.begin(); actionIt != actions.end(); ++actionIt)
    {
	previewAction = preview_viewer->actionCollection()->action(*actionIt);
	if (previewAction)
	    preview_viewer->actionCollection()->take(previewAction);
    }
    
    
}


klwLayout::~klwLayout()
{}


#include "klwlayout.moc"


void klwLayout::toggleDate(int i)
{
    kDateYourLetterFrom->setEnabled(i);
}

void klwLayout::handleTabChange(QWidget* m_Widget)
{
    currentPage = tabContainer->indexOf(m_Widget);
    tabContainer->setTabEnabled(tabContainer->page(currentPage+1),TRUE);

    if (currentPage == 2)
    {
        emit needNewPS();
        emit needGUIUpdating(TRUE);
	if ( ! previewIsUptodate )
	    preview_viewer->openURL(PSFile);
    }
    else
    {
        emit needGUIUpdating(FALSE);
    }


}

void klwLayout::settingsChanged()
{
    lineEditSalutation->setText(klwSettings::salutation());
    lineEditRegards->setText(klwSettings::regards());
}

void klwLayout::openPS(const KURL& url)
{
    if (currentPage == 2 )
	preview_viewer->openURL(url);
    PSFile = url;
}


void klwLayout::letterOpened(const klwLetter* l)
{
    address->lineEditName->setText(l->letterName);
    address->lineEditTo->setText(l->letterTo);
    address->lineEditStreet->setText(l->letterStreet);
    address->lineEditCity->setText(l->letterCity);
    lineEditYourSign->setText(l->letterYourSign);
    lineEditMySign->setText(l->letterMySign);
    if (l->yourLetterFromOn)
	chkYourLetterFrom->setChecked(TRUE);
    else
	chkYourLetterFrom->setChecked(FALSE);
    
    kDateYourLetterFrom->setDate(QDate::fromString(l->letterYourLetterFrom));
    lineEditSubject->setText(l->letterSubject);
    lineEditSalutation->setText(l->letterSalutation);
    lineEditEnclosures->setText(l->letterEnclosures);
    lineEditRegards->setText(l->letterRegards);
    textEditText->setText(l->letterText);
    tabContainer->setCurrentPage(0);
    emit needNewPS();
}




 
