/***************************************************************************
 *   Copyright (C) 2004 by Christoph                                       *
 *   chris@inferno.nadir.org                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "klwletter.h"
#include "klwSettings.h"
#include <kdebug.h>
#include <klocale.h>
#include <kglobal.h>
#include <kprocess.h>
#include <kfiledialog.h>
#include <kmessagebox.h>
#include <kio/job.h>

#include <qtextstream.h>
#include <qdom.h>

          

klwLetter::klwLetter(QObject *parent, const char *name)
 : QObject(parent, name)
{
    letterTmpDir = new KTempDir();
    letterDVIfile= new QFile(letterTmpDir->name() + "tmp.dvi");
    letterTexfile = new QFile(letterTmpDir->name() + "tmp.tex");
    letterPSfile = new QFile(letterTmpDir->name() + "tmp.ps");
    letterSalutation = klwSettings::salutation();
    letterRegards = klwSettings::regards();
    letterTEXisUptodate = FALSE;
    letterYourLetterFrom = QDate::currentDate().toString();
    yourLetterFromOn = FALSE;
}


klwLetter::~klwLetter()
{
    letterTmpDir->unlink();
}


#include "klwletter.moc"

void klwLetter::letterMakePS()
{
    if (! letterTEXisUptodate )
    {
	emit letterBuildingStart();
	letterMakeTex();
	letterTEXisUptodate = TRUE;
    }
}

QString klwLetter::escapeText(QString text)
{
	text.replace("\\","\\backslash");
	text.replace("#","\\#");
	text.replace("$","\\$");
	text.replace("%","\\%");
	//text.replace("^","\\^");
	text.replace("&","\\&");
	text.replace("_","\\_");
	//escapedText.replace("~","\\~");
	text.replace("{","$\\{$");
	text.replace("}","$\\}$");

	return text;
}

void klwLetter::letterMakeTex()
{
    if(letterTexfile->open(IO_WriteOnly))
    {
	
	


	QTextStream stream(letterTexfile);
	
	stream << "\\documentclass[11pt,ngerman]{klw}" << "\n";
	
        stream << "\\usepackage{babel}\n";
        
        stream << "\\usepackage{graphicx}\n";
	
	stream << "\\def\\datumtext{\\footnotesize\\textsc{" << i18n("Date") << "}}" << "\n";
	
	stream << "\\def\\telefontex{{\\footnotesize " << klwSettings::chooser1Text() << "\\/:}} \n";
	
	stream << "\\def\\telefaxtext{{\\footnotesize " << klwSettings::chooser2Text() << "\\/:}} \n";
	
	stream << "\\def\\emailtext{{\\footnotesize " << i18n("Email") << "\\/:}} \n";
	
	stream << "\\def\\httptext{{\\footnotesize " << i18n("URL") << "\\/:}} \n";
	
	stream << "\\def\\banktext{{\\footnotesize " << i18n("Bank") << "\\/:}} \n";
	
	stream << "\\def\\blztext{{\\footnotesize " << i18n("Bank Code") << "\\/:}} \n";
	
	stream << "\\def\\kontotext{{\\footnotesize " << i18n("Account Number") << "\\/:}} \n";
	
	if (letterYourSign == "")
		stream << "\\def\\ihrzeichentext{} \n";
	else
		stream << "\\def\\ihrzeichentext{{\\footnotesize " << i18n("Your Sign") << "}} \n";
	
	if (yourLetterFromOn)
		stream << "\\def\\ihrschreibentext{{\\footnotesize " << i18n("Your Letter from") << "}} \n";
	else
		stream << "\\def\\ihrschreibentext{} \n";
	
	
	if (letterMySign == "")
		stream << "\\def\\meinzeichentext{} \n";
	else
		stream << "\\def\\meinzeichentext{{\\footnotesize " << i18n("My Sign") << "}} \n";
	
	
	
	
	if (klwSettings::unicode())
	{
	    stream << "\\usepackage{ucs}\n";
	    stream << "\\usepackage[utf8]{inputenc}\n";
	}	
	else
	{
	    stream << "\\usepackage[latin1]{inputenc}\n";
	}
	
        stream << "\n";
	
	if (klwSettings::faltmarken())
            stream << "\\faltmarken" << "\n";
        if (klwSettings::fenstermarken())
            stream << "\\fenstermarken"<<"\n";
        if (klwSettings::trennlinien())
            stream << "\\trennlinien"<<"\n";
        if (klwSettings::lochermarken())
            stream << "\\lochermarke"<<"\n";

	stream << "\\Name{";
	if (klwSettings::title() != "")
                stream << klwSettings::title() << " ";
        stream << klwSettings::givenName() << " " << klwSettings::familyName() << "}\n";
        stream << "\\Strasse{" << klwSettings::street() << "}\n";
        if (klwSettings::zusatz() != "")
                stream << "\\Zusatz{" << klwSettings::zusatz() << "}\n";
        stream << "\\Ort{" << klwSettings::locality() << "}\n";

        stream << "\\Unterschrift{ }\n";

	if (! klwSettings::chooser2Value().isEmpty())
	    stream << "\\Telefax{"<< klwSettings::chooser2Value() << "}\n";
	if (! klwSettings::chooser1Value().isEmpty())
            stream << "\\Telefon{" << klwSettings::chooser1Value() << "}\n";
        if (klwSettings::checkEmail())
            stream << "\\EMail{" << klwSettings::email() << "}\n";
        if (klwSettings::checkURL())
            stream << "\\HTTP{" << klwSettings::uRL() << "}\n";
        if (klwSettings::checkBank())
	{
            stream << "\\Bank{" << klwSettings::bank() << "}\n";
            stream << "\\BLZ{" << klwSettings::bLZ() << "}\n";
	    stream << "\\Konto{" << klwSettings::ktoNr() << "}\n";
            }


	
	stream << "\\Betreff{" << escapeText(letterSubject) << "}\n";
        stream << "\\Datum{" << KGlobal::locale()->formatDate(QDate::currentDate(),1) << "}\n";
	
        if (letterYourSign != "")
            stream <<  "\\IhrZeichen{" << escapeText(letterYourSign) << "}\n";
	if (yourLetterFromOn)
	
	 stream << "\\IhrSchreiben{"<< KGlobal::locale()->formatDate(QDate::fromString(letterYourLetterFrom))<< "}\n";
	   
        stream << "\\Anrede{" << escapeText(letterSalutation) << "}\n";
        stream << "\\Adresse{" << escapeText(letterTo) << "\\strut\\\\";
        stream <<                 escapeText(letterName) << "\\\\ \n";
        stream <<                 escapeText(letterStreet) << " \\\\ \n";
        stream <<                 "\\\\ \n";
        stream <<                 escapeText(letterCity) << "\\\\} \n";


        stream << "\\Gruss{" << escapeText(letterRegards) << "}{1cm}\n";
	
	
	if ( letterMySign != "")
	    stream << "\\MeinZeichen{" << escapeText(letterMySign) << "}\n";
	if ( letterYourSign != "")
	    stream << "\\IhrZeichen{" << escapeText(letterYourSign) << "}\n";
	
	stream << "\\Anlagen{"<< escapeText(letterEnclosures) << "}\n";
	stream << "\\begin{document}\n";
        stream << "\\begin{klw}\\vspace{0.5cm}\n";
        stream << "\n";
        stream << escapeText(letterText) << "\n";
        stream << "\n";
        stream << "\\end{klw}"<<"\n";
        stream << "\\end{document}"<<"\n";
  	
	letterTexfile->close();
	letterMakeDVI();
    }
	


    
}

void klwLetter::letterMakeDVI()
{
    KProcess *proc = new KProcess;
    *proc << klwSettings::latex();
    *proc << " \\nonstopmode\\input\{" << letterTexfile->name() << "\}";
    proc->setWorkingDirectory(letterTmpDir->name());
    connect(proc,SIGNAL(processExited(KProcess* )),this,SLOT(letterBuildPS()));
    proc->start();
    
    
}

void klwLetter::letterBuildPS()
{
    KProcess *proc = new KProcess;
    *proc << klwSettings::dvips();
    *proc << letterDVIfile->name();
    *proc << "-o" + letterPSfile->name();
    connect(proc,SIGNAL(processExited(KProcess*)),this,SLOT(letterPSReady()));
    proc->start();
}

void klwLetter::letterPSReady()
{
    emit letterPSNew(KURL(letterPSfile->name()));
    emit letterBuildingEnd();
}
    
void klwLetter::somethingChanged()
{
    letterTEXisUptodate = FALSE;
}

void klwLetter::letterNewTo(const QString &to)
{
    letterTo = to;
    somethingChanged();
}

void klwLetter::letterNewName(const QString &name)
{
    letterName = name;
    somethingChanged();
}

void klwLetter::letterNewStreet(const QString &street)
{
    letterStreet = street;
    somethingChanged();
}

void klwLetter::letterNewCity(const QString &city)
{
    letterCity = city;
    somethingChanged();
}
 
void klwLetter::letterNewSalutation(const QString &salutation)
{
    letterSalutation = salutation;
    somethingChanged();
}

void klwLetter::letterNewRegards(const QString &regards)
{
    letterRegards = regards;
    somethingChanged();
}

void klwLetter::letterNewMySign(const QString &mySign)
{
    letterMySign = mySign;
    somethingChanged();
}

void klwLetter::letterNewYourSign(const QString &yourSign)
{
    letterYourSign = yourSign;
    somethingChanged();
}

void klwLetter::letterNewYourLetterFrom(QDate d)
{
    letterYourLetterFrom = d.toString();
    somethingChanged();
}

void klwLetter::letterNewEnclosures(const QString &enclosures)
{
    letterEnclosures = enclosures;
    somethingChanged();
}

void klwLetter::letterNewSubject(const QString &subject)
{
    letterSubject = subject;
    somethingChanged();
}

void klwLetter::letterNewText(QString text)
{
    letterText = text;
    somethingChanged();
}

void klwLetter::letterChkYourLetterFromChanged(int i)
{
    yourLetterFromOn = i;
    somethingChanged();
}


QString klwLetter::letterToXML()
{
    QDomDocument doc ("Letter");
    QDomElement root = doc.createElement("Letter");
    doc.appendChild(root);
 
    QDomElement tagName = doc.createElement("Name");
    root.appendChild(tagName);
    tagName.setAttribute("value",letterName);
 
    QDomElement tagTo = doc.createElement("To");
    root.appendChild(tagTo);
    tagTo.setAttribute("value",letterTo);
 
    QDomElement tagStreet = doc.createElement("Street");
    root.appendChild(tagStreet);
    tagStreet.setAttribute("value",letterStreet);
 
    QDomElement tagCity = doc.createElement("City");
    root.appendChild(tagCity);
    tagCity.setAttribute("value",letterCity);
 

    QDomElement tagYourSign =doc.createElement("YourSign");
    root.appendChild(tagYourSign);
    tagYourSign.setAttribute("value",letterYourSign);
 
    QDomElement tagMySign =doc.createElement("MySign");
    root.appendChild(tagMySign);
    tagMySign.setAttribute("value",letterMySign);
 
    QDomElement tagYourLetterFrom = doc.createElement("YourLetterFrom");
    root.appendChild(tagYourLetterFrom);
    if (yourLetterFromOn)
	tagYourLetterFrom.setAttribute("enabled","TRUE");
    else	
    tagYourLetterFrom.setAttribute("enabled","FALSE");
    QDomText yourLetterFromDate = doc.createTextNode(letterYourLetterFrom);
    tagYourLetterFrom.appendChild(yourLetterFromDate);
 
    QDomElement tagEnclosures = doc.createElement("Enclosures");
    root.appendChild(tagEnclosures);
    QDomText enclosureText = doc.createTextNode(letterEnclosures);
    tagEnclosures.appendChild(enclosureText);
 
    QDomElement tagSubject = doc.createElement("Subject");
    root.appendChild(tagSubject);
    tagSubject.setAttribute("value",letterSubject);
 
    QDomElement tagSalutation = doc.createElement("Salutation");
    root.appendChild(tagSalutation);
    tagSalutation.setAttribute("value",letterSalutation);
 
    QDomElement tagRegards = doc.createElement("Regards");
    root.appendChild(tagRegards);
    tagRegards.setAttribute("value",letterRegards);
 
    QDomElement tagText = doc.createElement("Text");
    root.appendChild(tagText);
 
    QDomText text = doc.createTextNode(letterText);
    tagText.appendChild(text);
    
    return doc.toCString();
}

void klwLetter::letterSave()
{

 if ( ! lmlFile.exists())
 {
     lmlFile.setName(KFileDialog::getSaveFileName("~","*.letter"));
     if (lmlFile.exists())
     {
            if (KMessageBox::questionYesNo(0,i18n("File already exists, overwrite ?")) == 4)
		lmlFile.setName(""); 
     }
     
 if (lmlFile.name() != "")
 {
 if(lmlFile.open(IO_WriteOnly))
    {
	QTextStream stream(&lmlFile);
	stream << letterToXML();
	lmlFile.close();
	kdDebug() << "File should be written now" << endl;
	emit fileSaved();
    }
 else
     KMessageBox::error(0,"Could Not open " + lmlFile.name() + " for writing","Error");
 }
 }
}

void klwLetter::letterSaveAs()
{
 
    //QString xml = letterToXML();
    lmlFile.setName(KFileDialog::getSaveFileName("~","*.letter"));
    if (lmlFile.name() != "")
    {
    if (lmlFile.open(IO_WriteOnly))
    {
	QTextStream stream(&lmlFile);
	stream << letterToXML();
	kdDebug() << "File should be written now" << endl;
	lmlFile.close();
	emit fileSaved();
    }
    else
	KMessageBox::error(0,"Could Not open " + lmlFile.name() + " for writing","Error");
    }
}

void klwLetter::letterSavePS()
{
    QString a;
    a = KFileDialog::getSaveFileName("~","*.ps");
    KIO::file_copy(KURL(letterPSfile->name()),KURL(a));
    
}

void klwLetter::letterSaveTex()
{
   QString a;
   a = KFileDialog::getSaveFileName("~","*.tex");
   KIO::file_copy(KURL(letterTexfile->name()),KURL(a));
}

void klwLetter::letterOpen()
{
    lmlFile.setName(KFileDialog::getOpenFileName()); 
    letterOpen(KURL(lmlFile.name()));
}

void klwLetter::letterOpen(KURL url)
{
 lmlFile.setName(url.path());
 
 QDomDocument doc("lmlFile");
 QFile file(lmlFile.name());
 
 if ( !file.open(IO_ReadOnly))
 {
     kdDebug() << "Could not open File for reading****************************************";
     return;
 }
 if(!doc.setContent(&file))
 {
	 file.close();
	 kdDebug() << "Could not parse File**************************************************";
	 return;
     }
 file.close();
 
 QDomElement docElem = doc.documentElement();
 QDomNodeList list;
 
 list = docElem.elementsByTagName("Name");
 letterName = list.item(0).toElement().attribute("value");
 
 list = docElem.elementsByTagName("To");
 letterTo = list.item(0).toElement().attribute("value");
 
 list = docElem.elementsByTagName("Street");
 letterStreet = list.item(0).toElement().attribute("value");
 
 list = docElem.elementsByTagName("City");
 letterCity = list.item(0).toElement().attribute("value");
 
 
 list = docElem.elementsByTagName("YourSign");
 letterYourSign = list.item(0).toElement().attribute("value");
 
 list = docElem.elementsByTagName("MySign");
 letterMySign = list.item(0).toElement().attribute("value");
 
 list = docElem.elementsByTagName("YourLetterFrom");
 if (list.item(0).toElement().attribute("enabled") == "TRUE")
     yourLetterFromOn = TRUE;
 else
     yourLetterFromOn = FALSE;
 
 letterYourLetterFrom = list.item(0).toElement().text();
 
 list = docElem.elementsByTagName("Enclosures");
 letterEnclosures = list.item(0).toElement().text();
 
 list = docElem.elementsByTagName("Subject");
 letterSubject = list.item(0).toElement().attribute("value");
 
 list = docElem.elementsByTagName("Salutation");
 letterSalutation = list.item(0).toElement().attribute("value");
 
 list = docElem.elementsByTagName("Regards");
 letterRegards = list.item(0).toElement().attribute("value");
 
 list = docElem.elementsByTagName("Text");
 letterText = list.item(0).toElement().text();
 
 emit letterLetterOpened(this);
 letterTEXisUptodate = FALSE;
}
